@@ -8,9 +8,9 @@
 
 #include <asiolink/testutils/timed_signal.h>
 #include <cc/command_interpreter.h>
+#include <d2srv/testutils/nc_test_utils.h>
 #include <d2/d2_controller.h>
 #include <d2/d2_process.h>
-#include <d2/tests/nc_test_utils.h>
 #include <process/testutils/d_test_stubs.h>
 
 #include <boost/pointer_cast.hpp>
