@@ -97,6 +97,9 @@ extern const isc::log::MessageID HA_STATE_MACHINE_CONTINUED;
 extern const isc::log::MessageID HA_STATE_MACHINE_PAUSED;
 extern const isc::log::MessageID HA_STATE_TRANSITION;
 extern const isc::log::MessageID HA_STATE_TRANSITION_PASSIVE_BACKUP;
+extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_COMMUNICATIONS_FAILED;
+extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_FAILED;
+extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_HANDLER_FAILED;
 extern const isc::log::MessageID HA_SYNC_FAILED;
 extern const isc::log::MessageID HA_SYNC_HANDLER_FAILED;
 extern const isc::log::MessageID HA_SYNC_START;
