@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -17,18 +17,17 @@ namespace dhcp {
 /// @brief Common configuration parser for shared networks
 /// and subnets.
 class BaseNetworkParser : public data::SimpleParser {
-protected:
+public:
 
-    /// @brief Parses DHCP lifetime.
+    /// @brief Moves deprecated reservation-mode parameter to
+    /// new reservations flags.
     ///
-    /// Used here for valid-lifetime but reused for preferred-lifetime.
-    ///
-    /// @param scope Data element holding e.g.  shared network configuration
-    /// to be parsed.
-    /// @param name Base name of the lifetime parameter.
-    /// @return A triplet with the parsed lifetime value.
-    const Triplet<uint32_t> parseLifetime(const data::ConstElementPtr& scope,
-                                          const std::string& name);
+    /// @param config [in/out] configuration to alter.
+    /// @throw DhcpConfigError on error e.g. when both reservation-mode
+    /// and a flag are specified.
+    static void moveReservationMode(isc::data::ElementPtr config);
+
+protected:
 
     /// @brief Parses common parameters
     ///
@@ -37,6 +36,9 @@ protected:
     /// - rebind-timer,
     /// - valid-lifetime,
     /// - store-extended-info
+    /// - reservations-global
+    /// - reservations-in-subnet
+    /// - reservations-out-of-pool
     ///
     /// @param network_data Data element holding shared network
     /// configuration to be parsed.
@@ -78,15 +80,6 @@ protected:
     void parseCacheParams(const data::ConstElementPtr& network_data,
                      NetworkPtr& network);
 
-    /// @brief Parses host reservation mode.
-    ///
-    /// @param network_data Data element holding shared network
-    /// configuration to be parsed.
-    /// @param [out] network Pointer to a network in which parsed data is
-    /// to be stored.
-    void parseHostReservationMode(const data::ConstElementPtr& network_data,
-                                  NetworkPtr& network);
-
     /// @brief Parses parameters pertaining to DDNS behavior.
     ///
     /// The parsed parameters are:
