@@ -1,4 +1,4 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,25 +6,27 @@
 
 #include <config.h>
 
+#include <testutils/gtest_utils.h>
 #include <yang/translator.h>
 
 #include <boost/scoped_ptr.hpp>
+
 #include <gtest/gtest.h>
 
 using namespace std;
 using namespace isc;
 using namespace isc::data;
 using namespace isc::yang;
-#ifndef HAVE_PRE_0_7_6_SYSREPO
 using namespace sysrepo;
-#endif
+
+using libyang::S_Data_Node;
 
 namespace {
 
 // Test constructor.
 TEST(TranslatorBasicTest, constructor) {
     // Get a connection.
-    S_Connection conn(new Connection("translator unittests"));
+    S_Connection conn(std::make_shared<Connection>());
     // Get a session.
     S_Session sess(new Session(conn, SR_DS_CANDIDATE));
     // Get a translator object.
@@ -61,7 +63,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Unsigned 8 bit integer.
     uint8_t u8(123);
-    s_val.reset(new Val(u8, SR_UINT8_T));
+    s_val.reset(new Val(u8));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -70,7 +72,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Unsigned 16 bit integer.
     uint16_t u16(12345);
-    s_val.reset(new Val(u16, SR_UINT16_T));
+    s_val.reset(new Val(u16));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -79,7 +81,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Unsigned 32 bit integer.
     uint32_t u32(123456789);
-    s_val.reset(new Val(u32, SR_UINT32_T));
+    s_val.reset(new Val(u32));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -88,7 +90,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Signed 8 bit integer.
     int8_t s8(-123);
-    s_val.reset(new Val(s8, SR_INT8_T));
+    s_val.reset(new Val(s8));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -97,7 +99,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Signed 16 bit integer.
     int16_t s16(-12345);
-    s_val.reset(new Val(s16, SR_INT16_T));
+    s_val.reset(new Val(s16));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -106,7 +108,7 @@ TEST(TranslatorBasicTest, valueFrom) {
 
     // Signed 32 bit integer.
     int32_t s32(-123456789);
-    s_val.reset(new Val(s32, SR_INT32_T));
+    s_val.reset(new Val(s32));
     EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
     ASSERT_TRUE(elem);
     ASSERT_EQ(Element::integer, elem->getType());
@@ -138,35 +140,25 @@ TEST(TranslatorBasicTest, valueFrom) {
     EXPECT_EQ("foobar", elem->stringValue());
     elem.reset();
 
-    // Unknown / unsupported.
+    // Decimal 64.
     double d64(.1234);
     s_val.reset(new Val(d64));
-    EXPECT_THROW(TranslatorBasic::value(s_val), NotImplemented);
+    EXPECT_NO_THROW(elem = TranslatorBasic::value(s_val));
+    ASSERT_TRUE(elem);
+    ASSERT_EQ(Element::real, elem->getType());
+    EXPECT_DOUBLE_EQ(d64, elem->doubleValue());
 }
 
 // Test basic yang value to JSON using sysrepo test models.
 TEST(TranslatorBasicTest, getItem) {
     // Get a translator object to play with.
-    S_Connection conn(new Connection("translator unittests"));
+    S_Connection conn(std::make_shared<Connection>());
     S_Session sess(new Session(conn, SR_DS_CANDIDATE));
     boost::scoped_ptr<TranslatorBasic> t_obj;
-    ASSERT_NO_THROW(t_obj.reset(new TranslatorBasic(sess, "")));
-
-    // Container.
-    string xpath = "/keatest-module:container/list";
+    ASSERT_NO_THROW_LOG(t_obj.reset(new TranslatorBasic(sess, "")));
     S_Val s_val;
-    EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     ConstElementPtr elem;
-    EXPECT_NO_THROW(elem = t_obj->getItem("/keatest-module:container"));
-    EXPECT_FALSE(elem);
-    elem.reset();
-
-    // List.
-    EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
-    ASSERT_TRUE(elem);
-    ASSERT_EQ(Element::list, elem->getType());
-    EXPECT_EQ(0, elem->size());
-    elem.reset();
+    string xpath;
 
     // String.
     xpath = "/keatest-module:main/string";
@@ -191,7 +183,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Unsigned 8 bit integer.
     xpath = "/keatest-module:main/ui8";
     uint8_t u8(8);
-    s_val.reset(new Val(u8, SR_UINT8_T));
+    s_val.reset(new Val(u8));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -202,7 +194,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Unsigned 16 bit integer.
     xpath = "/keatest-module:main/ui16";
     uint16_t u16(16);
-    s_val.reset(new Val(u16, SR_UINT16_T));
+    s_val.reset(new Val(u16));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -213,7 +205,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Unsigned 32 bit integer.
     xpath = "/keatest-module:main/ui32";
     uint32_t u32(32);
-    s_val.reset(new Val(u32, SR_UINT32_T));
+    s_val.reset(new Val(u32));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -224,7 +216,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Signed 8 bit integer.
     xpath = "/keatest-module:main/i8";
     int8_t s8(8);
-    s_val.reset(new Val(s8, SR_INT8_T));
+    s_val.reset(new Val(s8));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -235,7 +227,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Signed 16 bit integer.
     xpath = "/keatest-module:main/i16";
     int16_t s16(16);
-    s_val.reset(new Val(s16, SR_INT16_T));
+    s_val.reset(new Val(s16));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -246,7 +238,7 @@ TEST(TranslatorBasicTest, getItem) {
     // Signed 32 bit integer.
     xpath = "/keatest-module:main/i32";
     int32_t s32(32);
-    s_val.reset(new Val(s32, SR_INT32_T));
+    s_val.reset(new Val(s32));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
     ASSERT_TRUE(elem);
@@ -294,13 +286,13 @@ TEST(TranslatorBasicTest, getItem) {
 
     // Leaf-list: 1, 2 and 3.
     u8 = 1;
-    s_val.reset(new Val(u8, SR_UINT8_T));
+    s_val.reset(new Val(u8));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     u8 = 2;
-    s_val.reset(new Val(u8, SR_UINT8_T));
+    s_val.reset(new Val(u8));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     u8 = 3;
-    s_val.reset(new Val(u8, SR_UINT8_T));
+    s_val.reset(new Val(u8));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
     EXPECT_NO_THROW(elem = t_obj->getItems(xpath));
     ASSERT_TRUE(elem);
@@ -313,13 +305,18 @@ TEST(TranslatorBasicTest, getItem) {
     xpath = "/keatest-module:main/dec64";
     s_val.reset(new Val(9.85));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
-    EXPECT_THROW(elem = t_obj->getItem(xpath), NotImplemented);
+    EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
+    ASSERT_TRUE(elem);
+    ASSERT_EQ(Element::real, elem->getType());
+    EXPECT_EQ("9.85", elem->str());
     elem.reset();
 
     // Not found.
     xpath = "/keatest-module:main/no_such_string";
-    EXPECT_NO_THROW(sess->delete_item(xpath.c_str()));
-    EXPECT_NO_THROW(elem = t_obj->getItem(xpath));
+
+    // sysrepo_exception: "Invalid argument"
+    EXPECT_THROW(sess->delete_item(xpath.c_str()), sysrepo_exception);
+    EXPECT_THROW(elem = t_obj->getItem(xpath), SysrepoError);
     EXPECT_FALSE(elem);
     elem.reset();
 
@@ -329,7 +326,7 @@ TEST(TranslatorBasicTest, getItem) {
         elem = t_obj->getItem(xpath);
         ADD_FAILURE() << "expected exception";
     } catch (const SysrepoError& ex) {
-        EXPECT_EQ("sysrepo error getting item at 'null': Invalid argument",
+        EXPECT_EQ("sysrepo error getting item at 'null': libyang error",
                   string(ex.what()));
     } catch (const std::exception& ex) {
         ADD_FAILURE() << "unexpected exception with: " << ex.what();
@@ -441,18 +438,22 @@ TEST(TranslatorBasicTest, valueTo) {
     EXPECT_EQ("Zm9vYmFy", string(s_val->data()->get_binary()));
     s_val.reset();
 
-    // Unknown / unsupported.
-    elem = Element::create(.1234);
-    EXPECT_THROW(TranslatorBasic::value(elem, SR_DECIMAL64_T), NotImplemented);
+    // Decimal 64.
+    double d64(.1234);
+    elem = Element::create(d64);
+    EXPECT_NO_THROW(s_val = TranslatorBasic::value(elem, SR_DECIMAL64_T));
+    ASSERT_TRUE(s_val);
+    EXPECT_DOUBLE_EQ(d64, s_val->data()->get_decimal64());
+    s_val.reset();
 }
 
 // Test JSON to  basic yang value using sysrepo test models.
 TEST(TranslatorBasicTest, setItem) {
     // Get a translator object to play with.
-    S_Connection conn(new Connection("translator unittests"));
+    S_Connection conn(std::make_shared<Connection>());
     S_Session sess(new Session(conn, SR_DS_CANDIDATE));
     boost::scoped_ptr<TranslatorBasic> t_obj;
-    ASSERT_NO_THROW(t_obj.reset(new TranslatorBasic(sess, "")));
+    ASSERT_NO_THROW_LOG(t_obj.reset(new TranslatorBasic(sess, "")));
 
     // Container.
     string xpath = "/keatest-module:container";
@@ -461,17 +462,8 @@ TEST(TranslatorBasicTest, setItem) {
     EXPECT_THROW(t_obj->setItem(xpath, elem, SR_CONTAINER_PRESENCE_T),
                  NotImplemented);
 
-    // List.
-    xpath = "/keatest-module:container/list";
-    elem = Element::createList();
-    EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_LIST_T));
-    S_Val s_val;
-    EXPECT_NO_THROW(s_val = sess->get_item(xpath.c_str()));
-    ASSERT_TRUE(s_val);
-    ASSERT_EQ(SR_LIST_T, s_val->type());
-    s_val.reset();
-
     // String.
+    S_Val s_val;
     xpath = "/keatest-module:main/string";
     elem = Element::create(string("str"));
     EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_STRING_T));
@@ -581,35 +573,41 @@ TEST(TranslatorBasicTest, setItem) {
     EXPECT_EQ("Zm9vYmFy", string(s_val->data()->get_binary()));
     s_val.reset();
 
+    // Decimal 64.
+    xpath = "/keatest-module:main/dec64";
+    double d64(9.85);
+    elem = Element::create(d64);
+    EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_DECIMAL64_T));
+    EXPECT_NO_THROW(s_val = sess->get_item(xpath.c_str()));
+    ASSERT_TRUE(s_val);
+    ASSERT_EQ(SR_DECIMAL64_T, s_val->type());
+    EXPECT_DOUBLE_EQ(d64, s_val->data()->get_decimal64());
+    s_val.reset();
+
     // Leaf-list.
     xpath = "/keatest-module:main/numbers";
-    S_Vals s_vals;
-    EXPECT_NO_THROW(s_vals = sess->get_items(xpath.c_str()));
-    EXPECT_FALSE(s_vals);
-    s_vals.reset();
+    S_Data_Node data_node;
+    EXPECT_NO_THROW(data_node = sess->get_subtree(xpath.c_str()));
+    EXPECT_FALSE(data_node);
+    data_node.reset();
 
     // Fill it.
-    elem = Element::create(1);
-    EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_UINT8_T));
-    elem = Element::create(2);
-    EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_UINT8_T));
-    elem = Element::create(3);
-    EXPECT_NO_THROW(t_obj->setItem(xpath, elem, SR_UINT8_T));
-    EXPECT_NO_THROW(s_vals = sess->get_items(xpath.c_str()));
-    ASSERT_TRUE(s_vals);
-    EXPECT_EQ(3, s_vals->val_cnt());
-    s_vals.reset();
+    sess->set_item_str(xpath.c_str(), "1");
+    sess->set_item_str(xpath.c_str(), "2");
+    sess->set_item_str(xpath.c_str(), "3");
+    sess->apply_changes();
+
+    // sysrepo_exception: "Invalid argument"
+    EXPECT_THROW(data_node = sess->get_subtree(xpath.c_str()),
+        sysrepo_exception);
+    EXPECT_FALSE(data_node);
+    data_node.reset();
 
     // Clean it.
     EXPECT_NO_THROW(t_obj->delItem(xpath));
-    EXPECT_NO_THROW(s_vals = sess->get_items(xpath.c_str()));
-    EXPECT_FALSE(s_vals);
-    s_vals.reset();
-
-    // Unknown / unsupported.
-    xpath = "/keatest-module:main/dec64";
-    elem = Element::create(9.85);
-    EXPECT_THROW(t_obj->setItem(xpath, elem, SR_DECIMAL64_T), NotImplemented);
+    EXPECT_NO_THROW(data_node = sess->get_subtree(xpath.c_str()));
+    EXPECT_FALSE(data_node);
+    data_node.reset();
 
     // Bad xpath.
     xpath = "/keatest-module:main/no_such_string";
@@ -619,7 +617,7 @@ TEST(TranslatorBasicTest, setItem) {
         ADD_FAILURE() << "expected exception";
     } catch (const SysrepoError& ex) {
         string expected = "sysrepo error setting item '\"str\"' at '" +
-            xpath + "': Request contains unknown element";
+            xpath + "': Invalid argument";
         EXPECT_EQ(expected, string(ex.what()));
     } catch (const std::exception& ex) {
         ADD_FAILURE() << "unexpected exception with: " << ex.what();
@@ -630,56 +628,56 @@ TEST(TranslatorBasicTest, setItem) {
     elem = Element::create(true);
     try {
         t_obj->setItem(xpath, elem, SR_BOOL_T);
-        ADD_FAILURE() << "expected exception";
     } catch (const SysrepoError& ex) {
-        string expected = "sysrepo error setting item 'true' at '" +
-            xpath + "': Invalid argument";
-        EXPECT_EQ(expected, string(ex.what()));
+        ADD_FAILURE() << "unexpected exception with: " << ex.what();
     } catch (const std::exception& ex) {
         ADD_FAILURE() << "unexpected exception with: " << ex.what();
     }
 
+    // In sysrepo 1.x, set_item() is based on set_item_str() which sets the
+    // value in textual format. After setting a value with SR_BOOL_T, it's value
+    // is now "true". :)
+    elem = t_obj->getItem(xpath);
+    ASSERT_TRUE(elem);
+    EXPECT_EQ(elem->getType(), Element::string);
+    EXPECT_EQ(elem->str(), "\"true\"");
+
     // Delete (twice).
     xpath = "/keatest-module:main/string";
     EXPECT_NO_THROW(s_val = sess->get_item(xpath.c_str()));
     EXPECT_TRUE(s_val);
     s_val.reset();
     EXPECT_NO_THROW(t_obj->delItem(xpath));
-    EXPECT_NO_THROW(s_val = sess->get_item(xpath.c_str()));
+
+    // sysrepo_exception: "Item not found"
+    EXPECT_THROW(s_val = sess->get_item(xpath.c_str()), sysrepo_exception);
     EXPECT_FALSE(s_val);
     EXPECT_NO_THROW(t_obj->delItem(xpath));
 }
 
-// Test yang list iteration.
+// Test yang list item retrieval.
 TEST(TranslatorBasicTest, list) {
     // Get a translator object to play with.
-    S_Connection conn(new Connection("translator unittests"));
+    S_Connection conn(std::make_shared<Connection>());
     S_Session sess(new Session(conn, SR_DS_CANDIDATE));
     boost::scoped_ptr<TranslatorBasic> t_obj;
-    ASSERT_NO_THROW(t_obj.reset(new TranslatorBasic(sess, "")));
+    ASSERT_NO_THROW_LOG(t_obj.reset(new TranslatorBasic(sess, "")));
+    string xpath;
 
     // Empty list.
-    S_Iter_Value iter;
-    EXPECT_NO_THROW(iter = t_obj->getIter("/keatest-module:container/list"));
-    ASSERT_TRUE(iter);
-    string xpath;
-    EXPECT_NO_THROW(xpath = t_obj->getNext(iter));
-    EXPECT_TRUE(xpath.empty());
+    ElementPtr element;
+    EXPECT_NO_THROW(element = t_obj->getItem("/keatest-module:container/list"));
+    EXPECT_FALSE(element);
+    element.reset();
 
     // Retried with a filled list.
     xpath = "/keatest-module:container/list[key1='key1'][key2='key2']/leaf";
     S_Val s_val(new Val("Leaf value"));
     EXPECT_NO_THROW(sess->set_item(xpath.c_str(), s_val));
-    EXPECT_NO_THROW(iter = t_obj->getIter("/keatest-module:container/list"));
-    ASSERT_TRUE(iter);
-    EXPECT_NO_THROW(xpath = t_obj->getNext(iter));
-    EXPECT_EQ("/keatest-module:container/list[key1='key1'][key2='key2']",
-              xpath);
-    EXPECT_NO_THROW(xpath = t_obj->getNext(iter));
-    EXPECT_TRUE(xpath.empty());
-
-    // Not found: same than empty because sr_get_items_iter() translates
-    // SR_ERR_NOT_FOUND by SR_ERR_OK...
+    EXPECT_NO_THROW(element = t_obj->getItem("/keatest-module:container/list"));
+    ASSERT_TRUE(element);
+    EXPECT_NO_THROW(element = t_obj->getItem("/keatest-module:container/list[key1='key1'][key2='key2']"));
+    ASSERT_TRUE(element);
 }
 
-}; // end of anonymous namespace
+} // anonymous namespace
